# Clear environment
rm(list = ls())

# Librery
library(timelineS)
library(lubridate)
library(ggplot2)
library(dplyr)
library(reshape2)
library(rPref)
library(mco)
library(MASS)

rm(list = ls())


# Chart
plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {   
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) + years(lifetime))
  timelineS(dataP, main = plot.Name, labels = events.name, label.direction = "up", label.position = 3) 
}

# Events
dist.Events <- function(lifetime, events, start.Date, option.Name) {
  events <- sort(events, decreasing = TRUE)
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  all.events <- all.events[order(all.events$frequency), ]
  unique.events <- all.events[!duplicated(all.events$frequency), ]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
  # plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name)
  return(unique.events)
}

#combine.lifeTimelines <- function(product.1, product.2, p1.dur, p2.dur) {
#  base <- list(Names = c(), frequency = c(), duration = c())
#  base$frequency <- sort(unique(c(product.1$frequency, product.2$frequency)), decreasing = FALSE)
#  base$Names[1] <- "DC"
#  base$duration[1] <- 0
#  base$Names[length(base$frequency)] <- "END"
#  base$duration[length(base$frequency)] <- 0
#  
#  for(index in 2:(length(base$frequency) - 1)) {
#    phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])]
#    phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])]
#    
#    if(length(phase.1) != 0 && length(phase.2) != 0) {
#      base$Names[index] <- paste(phase.1, phase.2)
#      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)], 
#                                  p2.dur[which(names(p2.dur) == phase.2)])
#    } else if(length(phase.1) != 0 && length(phase.2) == 0) {
#      base$Names[index] <- phase.1
#      base$duration[index] <- p1.dur[which(names(p1.dur) == phase.1)]
#    } else if (length(phase.1) == 0 && length(phase.2) != 0) {
#      base$Names[index] <- phase.2
#      base$duration[index] <- p2.dur[which(names(p2.dur) == phase.2)]
#    } else {
#      base$duration[index] <- 0
#    }
#  }
#  return(base)
#}

# New combine
combine.lifeTimelines3 <- function(prod1, prod2, prod3, p1.dur, p2.dur, p3.dur) {
 
  base <- list(Names = c(), frequency = c(), duration = c())
  
  base$frequency <- sort(unique(c(prod1$frequency, prod2$frequency, prod3$frequency)), decreasing = FALSE)
  
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  
  base$Names[length(base$frequency)] <- "END"
  base$duration[length(base$frequency)] <- 0
  
  for(index in 2:(length(base$frequency) - 1)) {
    phase.1 <- prod1$event[which(prod1$frequency == base$frequency[index])]
    phase.2 <- prod2$event[which(prod2$frequency == base$frequency[index])]
    phase.3 <- prod3$event[which(prod3$frequency == base$frequency[index])]
    
    events_combined <- c()
    durations_combined <- c()
    
    if(length(phase.1) != 0) {
      events_combined <- c(events_combined, phase.1)
      durations_combined <- c(durations_combined, p1.dur[which(names(p1.dur) == phase.1)])
    }
    if(length(phase.2) != 0) {
      events_combined <- c(events_combined, phase.2)
      durations_combined <- c(durations_combined, p2.dur[which(names(p2.dur) == phase.2)])
    }
    if(length(phase.3) != 0) {
      events_combined <- c(events_combined, phase.3)
      durations_combined <- c(durations_combined, p3.dur[which(names(p3.dur) == phase.3)])
    }
    
    base$Names[index] <- paste(events_combined, collapse = " ")
    base$duration[index] <- ifelse(length(durations_combined) > 0, max(durations_combined), 0)
  }
  return(base)
}







start.Date <- "2025-01-01"
lifetime <- 80

# ==================== Original Maintenance Plans =================================================
# =================== Wind Turbines ====================================
events.Op1.WindTurbine <- c(
  PR.OWT = 40,
  M.OWT = 10,
  BR.OWT = 25,
  GR.OWT = 15,
  END = lifetime
)
duration.ev.WindTurbine <- c(
  PR.OWT = 15,
  M.OWT = 10,
  BR.OWT = 3,
  GR.OWT = 7
)

design.Options.WindTurbine <- list("Wind Turbine")


# =================== Offshore Structure ====================================
events.Op1.OStructure <- c(
  PR.OS = 35, 
  F.OS = 15, 
  M.OS = 5, 
  END.OS = lifetime
)
duration.ev.OStructure <- c(
  PR.OS = 20, 
  F.OS = 10, 
  M.OS = 10
)
design.Options.OStructure <- list("Offshore Structure Deck")


# =================== Breakwater ====================================
events.Op1.Breakwater <- c(
  M.B = 5,
  END.B = lifetime
)
duration.ev.Breakwater <- c(
  M.B = 13
)
design.Options.Breakwater <- list("Breakwater Standard Maintenance")

# Maintenance
maintenance.WindTurbine <- dist.Events(lifetime, events.Op1.WindTurbine, start.Date, design.Options.WindTurbine)
maintenance.OStructure   <- dist.Events(lifetime, events.Op1.OStructure, start.Date, design.Options.OStructure)
maintenance.Breakwater <- dist.Events(lifetime, events.Op1.Breakwater, start.Date, design.Options.Breakwater)

# Integration
integrated.interv.3 <- combine.lifeTimelines3(maintenance.WindTurbine, maintenance.OStructure, maintenance.Breakwater,
                                              duration.ev.WindTurbine, duration.ev.OStructure, duration.ev.Breakwater)
print(integrated.interv.3)
print(sum(integrated.interv.3$duration))




# ======================== Integrated Option 1 ===================================================
# =================== Wind Turbines ====================================
events.Op1.WindTurbine <- c(
  PR.OWT = 40,
  M.OWT = 5,
  BR.OWT = 25,
  GR.OWT = 15,
  END = lifetime
)
duration.ev.WindTurbine <- c(
  PR.OWT = 15,
  M.OWT = 5,
  BR.OWT = 3,
  GR.OWT = 7
)

design.Options.WindTurbine <- list("Wind Turbine")


# =================== Offshore Structure ====================================
events.Op1.OStructure <- c(
  PR.OS = 35, 
  F.OS = 25, 
  M.OS = 5, 
  END.OS = lifetime
)
duration.ev.OStructure <- c(
  PR.OS = 20, 
  F.OS = 7, 
  M.OS = 10
)
design.Options.OStructure <- list("Offshore Structure Deck")


# =================== Breakwater ====================================
events.Op1.Breakwater <- c(
  M.B = 5,
  END.B = lifetime
)
duration.ev.Breakwater <- c(
  M.B = 5
)
design.Options.Breakwater <- list("Breakwater Standard Maintenance")

# Maintenance
maintenance.WindTurbine <- dist.Events(lifetime, events.Op1.WindTurbine, start.Date, design.Options.WindTurbine)
maintenance.OStructure   <- dist.Events(lifetime, events.Op1.OStructure, start.Date, design.Options.OStructure)
maintenance.Breakwater <- dist.Events(lifetime, events.Op1.Breakwater, start.Date, design.Options.Breakwater)

# Integration
integrated.interv.3 <- combine.lifeTimelines3(maintenance.WindTurbine, maintenance.OStructure, maintenance.Breakwater,
                                              duration.ev.WindTurbine, duration.ev.OStructure, duration.ev.Breakwater)
print(integrated.interv.3)
print(sum(integrated.interv.3$duration))




# ======================== Integrated Option 2 ===================================================
# =================== Wind Turbines ====================================
events.Op1.WindTurbine <- c(
  PR.OWT = 40,
  M.OWT = 5,
  BR.OWT = 25,
  GR.OWT = 15,
  END = lifetime
)
duration.ev.WindTurbine <- c(
  PR.OWT = 15,
  M.OWT = 5,
  BR.OWT = 3,
  GR.OWT = 7
)

design.Options.WindTurbine <- list("Wind Turbine")


# =================== Offshore Structure ====================================
events.Op1.OStructure <- c(
  PR.OS = 40, 
  F.OS = 25, 
  M.OS = 5, 
  END.OS = lifetime
)
duration.ev.OStructure <- c(
  PR.OS = 15, 
  F.OS = 7, 
  M.OS = 10
)
design.Options.OStructure <- list("Offshore Structure Deck")

# =================== Breakwater ====================================
events.Op1.Breakwater <- c(
  M.B = 5,
  END.B = lifetime
)
duration.ev.Breakwater <- c(
  M.B = 5
)
design.Options.Breakwater <- list("Breakwater Standard Maintenance")

# Maintenance
maintenance.WindTurbine <- dist.Events(lifetime, events.Op1.WindTurbine, start.Date, design.Options.WindTurbine)
maintenance.OStructure   <- dist.Events(lifetime, events.Op1.OStructure, start.Date, design.Options.OStructure)
maintenance.Breakwater <- dist.Events(lifetime, events.Op1.Breakwater, start.Date, design.Options.Breakwater)

# Integration
integrated.interv.3 <- combine.lifeTimelines3(maintenance.WindTurbine, maintenance.OStructure, maintenance.Breakwater,
                                              duration.ev.WindTurbine, duration.ev.OStructure, duration.ev.Breakwater)
print(integrated.interv.3)
print(sum(integrated.interv.3$duration))





# =================== Automate the exploration ====================================
# New explore fuction
design.explore3 <- function(events1, events2, events3) {
  results <- c()
  
  for(i in 1:dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, design.Options.WindTurbine)
    dur.ev1 <- ev1 / 2
    
    for(j in 1:dim(events2)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, design.Options.OStructure)
      dur.ev2 <- ev2 / 2
      
      for(k in 1:dim(events3)[1]) {
        ev3 <- unlist(events3[k, ])
        dist.3 <- dist.Events(lifetime, ev3, start.Date, design.Options.Breakwater)
        dur.ev3 <- ev3 / 2
        
        combined.lifetime <- combine.lifeTimelines3(dist.1, dist.2, dist.3,
                                                    dur.ev1, dur.ev2, dur.ev3)
        
        total.dur <- sum(combined.lifetime$duration)
        #min.dist.int <- min(abs(diff(combined.lifetime$frequency)))
        min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] - combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
        
        results <- rbind(results, c(ev1, ev2, ev3, dur = total.dur, dist.inter = min.dist.int))
      }
    }
  }
  return(as.data.frame(results))
}


n.grid = 3  # n.grid random sample we create

# Wind Turbine
events.grid.WindTurbine = expand.grid(
  PR.OWT = sample(seq(35, 45, by = 1), n.grid), 
  M.OWT = sample(seq(5, 10, 1), n.grid),
  BR.OWT = sample(seq(25, 35, 1), n.grid),
  GR.OWT = sample(seq(5, 15, 1), n.grid)
)

# Offshore_Structure
events.grid.OStructure = expand.grid(
  PR.OS = sample(seq(30, 40, by = 1), n.grid), 
  F.OS = sample(seq(10, 20, 1), n.grid), 
  M.OS = sample(seq(5, 10, 1), n.grid)
)

# Offshore_Structure
events.grid.Breakwater = expand.grid(
  M.B = sample(seq(5, 12, by = 1), n.grid)
)

# ====== Option 2 breakwater ===================

# Wind Turbine
#events.grid.WindTurbine = expand.grid(
#  PR.OWT = sample(seq(35, 45, by = 1), n.grid), 
#  M.OWT = sample(seq(5, 10, 1), n.grid),
#  #BR.OWT = sample(seq(20, 30, 1), n.grid),
#  GR.OWT = sample(seq(5, 15, 1), n.grid)
#)
#
## Offshore_Structure
#events.grid.OStructure = expand.grid(
#  PR.OS = sample(seq(30, 40, by = 1), n.grid), 
#  F.OS = sample(seq(10, 20, 1), n.grid), 
#  M.OS = sample(seq(3, 10, 1), n.grid)
#)
#
## Offshore_Structure
#events.grid.Breakwater = expand.grid(
#  M.B = sample(seq(5, 12, by = 1), n.grid),
#  PR.B = sample(seq(7, 12, by = 1), n.grid)
#)


# Pareto
response.space = design.explore3(events.grid.WindTurbine, events.grid.OStructure, events.grid.Breakwater)
p = low(dur) * high(dist.inter)
pareto2 = psel(response.space, p, top = nrow(response.space))

ggplot(response.space, aes(x = dur, y = dist.inter)) + 
  geom_point(shape = 21) + 
  geom_point(data = pareto2, size = 3, aes(color = factor(.level))) +
  labs(title = "Exploration for 3 Options", x = "Duration", y = "Distance Between Events") +
  theme_minimal()

show_front = function(pref) { 
  plot(response.space$dur, response.space$dist.inter) 
  sky = psel(response.space, pref) 
  plot_front(response.space, pref, col = rgb(0, 0, 1)) 
  points(sky$dur, sky$dist.inter, lwd = 3) 
}

show_front(p)
p = high(dur) * low(dist.inter) 
show_front(p)







# --- Life Cycle Inventory and Analysis ---

# Fuction: Energy consumed and emissions generated.
LCI.materials <- read.csv("C:/Users/llame/OneDrive/Escritorio/R files/LCIMaterials_GroupProject.csv")

# Breakwater
LCA.Breakwater = function(indivsystem, materials, dist.event) {
  
  # Dimensions
  
  # create subsets for different scopes
  materials.split = split(materials, materials$scope)
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.event$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors = FALSE, row.names = names(s)) 
  
  # Calculations of the Materials
  if (indivsystem == "Breakwater") {
    concrete.int.volume = 0.015
    interventions.breakwater <- ifelse("Start" %in% colnames(s2), s2$Start, 0) +
      0.55 * ifelse("M.B" %in% colnames(s2), s2$M.B, 0)
  }
      
  # Create LCA matrix
  Breakwater <- mutate(materials.split[[indivsystem]], Combined.Q = quantities * concrete.int.volume, interventions = interventions.breakwater)
  
  # LCA Matrix = all the elements based on the options we defined.
  LCA.matrix <- rbind(Breakwater)
  LCA.matrix <- mutate(LCA.matrix, 
                       materials.LC = quantities + Combined.Q * interventions, # t materials
                       energy.LC = materials.LC * energy, 
                       CO2.LC = materials.LC * CO2,
                       NOX.LC = materials.LC * NOX,
                       SO2.LC = materials.LC * SO2)
    
  # Return results
  LCA.results <- list(
    energy = sum(LCA.matrix$energy.LC),
    CO2 = sum(LCA.matrix$CO2.LC), # index in kg emissions
    NOX = sum(LCA.matrix$NOX.LC), # index in kg emissions
    SO2 = sum(LCA.matrix$SO2.LC) # index in kg emissions
  )
  #print(str(LCA.results))
  return(LCA.results)
}
  
Option.Breakwater <- LCA.Breakwater(
  indivsystem = "Breakwater",
  materials = LCI.materials, 
  dist.event = maintenance.Breakwater
)
#print(Option.Breakwater)


# Offshore structures
LCA.OStructure <- function(indivsystem, materials, dist.event){
  
  # Dimensions
  S355.density <- 7850  # unit: kg/m3
  offshoredeckarea <- 50*70 # units: m 
  
  # create subsets for different scopes
  materials.split = split(materials, materials$scope)
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.event$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors = FALSE, row.names = names(s)) 
  
  
  # Calculations of the Materials
  if (indivsystem == "OffshoreGrating") {
    steelgrating <- offshoredeckarea* 12.56 * S355.density #12.56 is steel grating weight per sq meter
    structuralsteel <- 0
    interventions.offshore <- 
      ifelse(!is.null(s2$Start), s2$Start, 0) + 
      ifelse(!is.null(s2$M.OS), s2$M.OS, 0) + 
      0.25 * ifelse(!is.null(s2$F.OS), s2$F.OS, 0) + 
      0.35 * ifelse(!is.null(s2$PR.OS), s2$PR.OS, 0)
  } 
  else if (indivsystem == "OffshoreGirder") {
    structuralsteel <- offshoredeckarea* S355.density *106 /1.5 #every 1.5 meter on the long side with 50 meter length
    steelgrating <- 0
    interventions.offshore <- 
      ifelse(!is.null(s2$Start), s2$Start, 0) + 
      ifelse(!is.null(s2$M.OS), s2$M.OS, 0) + 
      0.25 * ifelse(!is.null(s2$F.OS), s2$F.OS, 0) + 
      0.35 * ifelse(!is.null(s2$PR.OS), s2$PR.OS, 0)
  }

  
  # Create LCA matrix
  OffshoreGrating <- mutate(materials.split[[indivsystem]], Combined.Q = steelgrating, interventions = interventions.offshore)
  OffshoreGirder <- mutate(materials.split[[indivsystem]], Combined.Q = structuralsteel, interventions = interventions.offshore)
  
  # LCA Matrix = all the elements based on the options we defined.
  LCA.matrix <- rbind(OffshoreGrating, OffshoreGirder)
  LCA.matrix <- mutate(
    LCA.matrix,
    TotalMaterials.Q = quantities * Combined.Q / 1000, # ton
    materials.LC = TotalMaterials.Q * interventions,
    energy.LC = materials.LC * energy,
    CO2.LC = materials.LC * CO2 * 1000, # index in g emissions
    NOX.LC = materials.LC * NOX * 1000, # index in g emissions
    SO2.LC = materials.LC * SO2 * 1000) # index in g emissions
  
  # Return results
  LCA.results <- list(
    energy = sum(LCA.matrix$energy.LC),
    CO2 = sum(LCA.matrix$CO2.LC),
    NOX = sum(LCA.matrix$NOX.LC),
    SO2 = sum(LCA.matrix$SO2.LC) # kg emissions / kg material
  )
  
  return(LCA.results)
}


Option.OffshoreGrating <- LCA.OStructure(
  indivsystem = "OffshoreGrating", 
  materials = LCI.materials, 
  dist.event = maintenance.OStructure
)
#print(Option.OffshoreGrating)

Option.OffshoreGirder <- LCA.OStructure(
  indivsystem = "OffshoreGirder",
  materials = LCI.materials,
  dist.event = maintenance.OStructure
)
#print(Option.OffshoreGirder)






# Wind turbines
LCA.WindTurbine = function(indivsystem, materials, dist.event) {
  
  # Dimensions
  S355.density <- 7850  # unit: kg/m3
  
  # create subsets for different scopes
  materials.split = split(materials, materials$scope)
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.event$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors = FALSE, row.names = names(s)) 
  
  # Calculations of the Materials
  if (indivsystem == "Wind") {
    C40Concrete <- 39
    structuralsteel <- 0
    interventions.wind <- ifelse("Start" %in% colnames(s2), s2$Start, 0) +
      ifelse("M.OWT" %in% colnames(s2), s2$M.OWT, 0) +
      0.55 * ifelse("BR.OWT" %in% colnames(s2), s2$BR.OWT, 0)
    
  } else if (indivsystem == "WindJacket") {
    structuralsteel <- 115.48 * S355.density 
    C40Concrete <- 0
    interventions.wind <- ifelse("Start" %in% colnames(s2), s2$Start, 0) +
      ifelse("M.OWT" %in% colnames(s2), s2$M.OWT, 0) +
      0.55 * ifelse("BR.OWT" %in% colnames(s2), s2$BR.OWT, 0)
  }
  
  # Create LCA matrix
  Wind <- mutate(materials.split[[indivsystem]], Combined.Q = C40Concrete, interventions = interventions.wind)
  WindJacket <- mutate(materials.split[[indivsystem]], Combined.Q = structuralsteel, interventions = interventions.wind)
  
  # LCA Matrix = all the elements based on the options we defined.
  LCA.matrix <- rbind(Wind, WindJacket)
  LCA.matrix <- mutate(
    LCA.matrix,
    TotalMaterials.Q = quantities * Combined.Q / 1000,
    materials.LC = TotalMaterials.Q * interventions,
    energy.LC = materials.LC * energy,
    CO2.LC = materials.LC * CO2 * 1000,
    NOX.LC = materials.LC * NOX * 1000,
    SO2.LC = materials.LC * SO2 * 1000)
  
  # Return results
  LCA.results <- list(
    energy = sum(LCA.matrix$energy.LC),
    CO2 = sum(LCA.matrix$CO2.LC),
    NOX = sum(LCA.matrix$NOX.LC),
    SO2 = sum(LCA.matrix$SO2.LC)
  )

  return(LCA.results)
}

Option.Wind <- LCA.WindTurbine(
  indivsystem = "Wind",
  materials = LCI.materials,
  dist.event = maintenance.WindTurbine
)
#print(Option.Wind)

Option.WindJacket <- LCA.WindTurbine(
  indivsystem = "WindJacket",
  materials = LCI.materials, 
  dist.event = maintenance.WindTurbine
)
#print(Option.WindJacket)



integrated.Design <- as.data.frame(list(
  Energy = Option.OffshoreGrating$energy + Option.OffshoreGirder$energy + 
    Option.Wind$energy + Option.WindJacket$energy + Option.Breakwater$energy,
  
  CO2 = Option.OffshoreGrating$CO2 + Option.OffshoreGirder$CO2 + 
    Option.Wind$CO2 + Option.WindJacket$CO2 + Option.Breakwater$CO2,
  
  NOx = Option.OffshoreGrating$NOX + Option.OffshoreGirder$NOX + 
    Option.Wind$NOX + Option.WindJacket$NOX + Option.Breakwater$NOX,
  
  SO2 = Option.OffshoreGrating$SO2 + Option.OffshoreGirder$SO2 + 
    Option.Wind$SO2 + Option.WindJacket$SO2 + Option.Breakwater$SO2
))
#print(integrated.Design)

# Costs
Energy.costs <- 0.128 
CO2.unitcost <- 26 # per metric tone 
NOx.unitCost <- 42 # per metric tone 
SO2.unitCosts <- 85 # per metric tone

integrated.Design <- mutate(integrated.Design, 
                            Costs = (Energy * Energy.costs + CO2*CO2.unitcost + NOx * NOx.unitCost + SO2 * SO2.unitCosts)/10^9)

#print(integrated.Design)









fitness3 <- function(x) {
  z <- numeric(7)
  x <- round(x, 0)
  
  # See events
  y <<- expand.grid(PR.OWT = x[1],  M.OWT = x[2],  BR.OWT = x[3],  GR.OWT = x[4],
                    PR.OS = x[5],  F.OS = x[6],  M.OS = x[7],
                    M.B = x[8]
                    )
  
  # Duration
  dur.ev1 <- unlist(y[1:4] / 2)
  dur.ev2 <- unlist(y[5:7] / 2)
  dur.ev3 <- unlist(y[8] / 2)
  
  # Generar las líneas de tiempo para cada sistema
  dist.1 <- apply(y[1:4], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.WindTurbine)
  dist.2 <- apply(y[5:7], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.OStructure)
  dist.3 <- apply(y[8], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.Breakwater)
  
  results <- combine.lifeTimelines3(dist.1[[1]], dist.2[[1]], dist.3[[1]], dur.ev1, dur.ev2, dur.ev3)
  
  z[1] <- sum(results[["duration"]])
  z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  
  #LCA:
  product.Wind <- LCA.WindTurbine(indivsystem = "Wind", materials = LCI.materials, dist.event = dist.1[[1]])
  product.WindJacket <- LCA.WindTurbine(indivsystem = "WindJacket", materials = LCI.materials, dist.event = dist.1[[1]])

  product.OffshoreGrating <- LCA.OStructure(indivsystem = "OffshoreGrating", materials = LCI.materials, dist.event = dist.2[[1]])
  product.OffshoreGirder <- LCA.OStructure(indivsystem = "OffshoreGirder", materials = LCI.materials, dist.event = dist.2[[1]])
  
  product.Breakwater <- LCA.Breakwater(indivsystem = "Breakwater", materials = LCI.materials, dist.event = dist.3[[1]])
  
  #Integration
  integrated.system <- as.data.frame(list(
    Energy = product.OffshoreGrating$energy + product.OffshoreGirder$energy + 
      product.Wind$energy + product.WindJacket$energy + product.Breakwater$energy,
    
    CO2 = product.OffshoreGrating$CO2 + product.OffshoreGirder$CO2 + 
      product.Wind$CO2 + product.WindJacket$CO2 + product.Breakwater$CO2,
    
    NOx = product.OffshoreGrating$NOX + product.OffshoreGirder$NOX + 
      product.Wind$NOX + product.WindJacket$NOX + product.Breakwater$NOX,
    
    SO2 = product.OffshoreGrating$SO2 + product.OffshoreGirder$SO2 + 
      product.Wind$SO2 + product.WindJacket$SO2 + product.Breakwater$SO2
  ))
  
  # Costs
  Energy.costs <- 0.128 
  CO2.unitcost <- 26 # per metric tone 
  NOx.unitCost <- 42 # per metric tone 
  SO2.unitCosts <- 85 # per metric tone
  
  integrated.system <- mutate(integrated.system, 
                              Costs = (Energy * Energy.costs + CO2*CO2.unitcost + NOx * NOx.unitCost + SO2 * SO2.unitCosts)/10^9)
  
  z[3] <- integrated.system$Energy 
  z[4] <- integrated.system$CO2 
  z[5] <- integrated.system$NOx 
  z[6] <- integrated.system$SO2 
  z[7] <- integrated.system$Costs
  return(z)
}

# Considering the ranges
lower_bounds <- c(35, 5, 20, 5, 30, 10, 3, 5)
upper_bounds <- c(45, 10, 30, 15, 40, 20, 10, 12)

# idim = input dimension = maintenance 
# odim = output dimension 
#ADD EDIT IDIM WITH NICOLAS ROLO MAINTENANCE VALUES
#Mo is also varying the bridge width 
# Run NSGA-II optimization
r2 <- nsga2(fitness3, idim = 8, odim = 7, generations=10, popsize=100, lower.bounds = lower_bounds, upper.bounds = upper_bounds)

# Transform results to dataframe
r2Results <- as.data.frame(r2$value) 
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" ) 
colnames(r2Results) <- outNames

# Extract Pareto front
pareto3 <- as.data.frame(paretoFront(r2)) 
colnames(pareto3) <- outNames

# Plot Pareto front (duration vs cost)
ggplot(r2Results, aes(x = duration, y = cost)) + 
  geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") + 
  geom_line(data = pareto3, color="blue") +
  labs(title = "Pareto Front: Duration vs Cost",
       x = "Total Duration of Interventions",
       y = "Total Cost")




# Prepare data for parallel coordinate plot
input.params <- round(r2$par, 0)
all.results <- cbind(input.params, r2Results, r2$pareto.optimal)
colnames(all.results) <- c(
  "PR.OWT", "M.OWT", "BR.OWT", "GR.OWT",
  "PR.OS", "F.OS", "M.OS",
  "M.B",
  "duration", "interv.dist",
  "energy", "co2", "nox", "so2", "cost",
  "pareto"
)


par(mfrow = c(1, 1))
parcoord(
  all.results[, 1:15], var.label = TRUE,
  col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"),
  lty = ifelse(all.results$pareto == TRUE, 1, 3),
  lwd = ifelse(all.results$pareto == TRUE, 3, 1)
)
title("Parallel Coordinate Plot of Design Parameters and Objectives")



#install.packages("GGally")
library(GGally)

# Crear el gráfico con normalización min-max
ggparcoord(
  data = all.results, 
  columns = 1:15,         # Selecciona las columnas a graficar
  groupColumn = "pareto", # Agrupa por si pertenece al Frente de Pareto
  scale = "uniminmax",    # Normaliza los valores entre 0 y 1
  alphaLines = 0.5        # Transparencia de las líneas
) +
  scale_color_manual(values = c("skyblue2", "indianred")) + 
  scale_linetype_manual(values = c(3, 1)) + 
  labs(title = "Parallel Coordinate Plot of Design Parameters and Objectives") +
  theme_minimal()


# Print summary of Pareto-optimal solutions
cat("\nSummary of Pareto-optimal solutions:\n")
print(summary(pareto3))

# Print best solution for each objective
cat("\nBest solutions for each objective:\n")
for(obj in outNames) {
  if(obj == "interv.dist") {
    best <- pareto3[which.max(pareto3[[obj]]),]
  } else {
    best <- pareto3[which.min(pareto3[[obj]]),]
  }
  cat(paste0("Best ", obj, ":\n"))
  print(best)
  cat("\n")
}

# Save results
write.csv(pareto3, "pareto_optimal_solutions.csv", row.names = FALSE)



z_results <- r2$value  # Extract the full matrix of z values

# Extract individual components for clarity (optional)
z1_results <- z_results[, 1]  # Duration (z[1])
z2_results <- z_results[, 2]  # Intervention Distance (z[2])
z3_results <- z_results[, 3]  # Energy (z[3])
z4_results <- z_results[, 4]  # CO2 (z[4])
z5_results <- z_results[, 5]  # NOx (z[5])
z6_results <- z_results[, 6]  # SO2 (z[6])
z7_results <- z_results[, 7]  # Cost (z[7])

# Display the first few results for all z-values
head(z_results)

# Display the first few entries of each result
head(z1_results)
head(z2_results)
head(z3_results)
head(z4_results)
head(z5_results)
head(z6_results)
head(z7_results)

# Display summaries for all z-values
summary(z_results)

# Individual summaries (if needed for specific variables)
summary(z1_results)
summary(z2_results)
summary(z3_results)
summary(z4_results)
summary(z5_results)
summary(z6_results)
summary(z7_results)

summary(r2Results)
head(pareto3)




